import string

class Schema:

    def __init__(self,schema) -> None:
        self.__schema = schema

    def __getitem__(self, field):
        return self.__schema[field]

    def keys(self):
        return self.__schema.keys()
        
    def is_translatable(self,field):
        if 'translatable' in self.__schema[field]:
            return self.__schema[field]['translatable']
        else:
            return True        

    def needs_spec(self,field):
        if 'spec' in self.__schema[field]:
            return self.__schema[field]
        else:
            return False
 
    def get_value(self,meta,field,lang):
        """
        treats receiving the value like a variant, 
        returns values with their language specific representations
        """
        match self.__schema[field]['type']: 
            case 'str': return meta[field][lang] if self.is_translatable(field) else meta[field]['value']
            case 'enum' | 'int' | 'num' | 'multikey' : return meta[field]['value']
            case 'multinum': return meta[field]['value'] if hasattr(meta[field]['value'],'__iter__') else [meta[field]['value'],] # force list!
    
    def to_list_of_dict(self,meta,fields,lang):
        """
        generates a list of dict which can easily be converted 
        to a pandas dataframe
        """
        # list comprehension for rows
        return [{'field' : field,   # field name
                  'lang' : lang,    # language shortcode
                  'type' : self.__schema[field]['type'], # datatype
                  'label' : self.__schema[field]['label'][lang], # label
                  'value' : self.get_value(meta,field,lang), # actual value
                  'template' : self.__schema[field]['template'][lang] if 'template' in self.__schema[field] else None,
                  # getting crazy with nested dict comprehension
                  'enum_values' : { k:v[lang] for (k,v) in self.__schema[field]['values'].items()} if 'enum' in self.__schema[field]['type'] else None,
                  'key_values' : { k:v[lang] for (k,v) in self.__schema[field]['keys'].items()} if 'multikey' in self.__schema[field]['type'] else None,
                  'spec' : meta[field]['spec'][lang] if 'spec' in meta[field] else None
                  }
                  for field in fields]
    

    def to_short_dict(self,meta,fields,lang):
        """
        generates a short version of dict which can easily be converted 
        to a pandas dataframe
        """
        # dict comprehension for whole meta part
        return { field : self.get_value(meta,field,lang) for field in fields }
    
    def to_list_of_tuple(self,meta,fields,lang):
        """
        generates a list of tuples with a label and value (text)
        this is usually consumed by a Markdown generator

        todo: needs deuglyfication of free standing loop, templates are possible for all
        """
        list = []
        for r in self.to_list_of_dict(meta,fields,lang):
            match r['type']:
                case 'str' : 
                    list.append( (r['label'],r['value']) )
                case 'int' | 'num' : 
                    list.append( ( r['label'], r['template'].format(value=r['value'],spec=r['spec']) if r['template'] else r['value']) )
                case 'enum' : 
                    list.append( ( r['label'], r['template'].format(value=r['enum_values'][r['value']],spec=r['spec']) 
                                  if r['template'] else r['enum_values'][r['value']] ) )
                case 'multikey' : 
                    list.append( ( r['label'], ', '.join( [r['template'].format(key=r['key_values'][k],value=v) for k,v in r['value'].items()] ) ) )
                case 'multinum' :
                    list.append( (r['label'], ', '.join( r['template'].format(value=v) for v in r['value'])) )

        return list

